/*
 * Copyright 1999, Alexander Feldman <alex@varna.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Alexander Feldman nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ALEXANDER FELDMAN AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL ALEXANDER FELDMAN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "tokenize.hpp"

char **add_string(char **, char *, int, int *);

char **tokenize(char *pszString, char *pszDelimiters)
{
	char **ppszResult = (char **)NULL;
	int iStrings = 0;
	
	assert(pszString);
	assert(pszDelimiters);
	
	char *pszStartOfToken;
	char *pszEndOfToken;
	
	for (pszStartOfToken = pszEndOfToken = pszString; *pszString != '\0'; pszString++)
		for (int j = 0; pszDelimiters[j] != '\0'; j++)
			if (*pszString == pszDelimiters[j]) {
				pszEndOfToken = pszString;
				ppszResult = add_string(ppszResult, pszStartOfToken, pszEndOfToken - pszStartOfToken, &iStrings);
				pszStartOfToken = pszString + 1;
			}
	pszEndOfToken = pszString;
	ppszResult = add_string(ppszResult, pszStartOfToken, pszEndOfToken - pszStartOfToken, &iStrings);
	ppszResult = add_string(ppszResult, NULL, 0, &iStrings);
	
	return ppszResult;
}

char **add_string(char **ppszList, char *pszString, int iLength, int *piStrings)
{
	char **ppszResult;
	char *pszCopy = pszString;
	
	if (NULL != pszString) {
		pszCopy = new char[iLength + 1];
		if (NULL != pszCopy) {
			memcpy(pszCopy, pszString, iLength);
			pszCopy[iLength] = '\0';
		}
	}
	if (NULL != pszCopy || NULL == pszString) {
		ppszResult = (char **)realloc(ppszList, sizeof(char *) * (*piStrings + 1));
		if (NULL != ppszResult) {
			ppszList = ppszResult;
			ppszResult[*piStrings] = pszCopy;
			*piStrings += 1;
		}
	}
	return ppszList;
}