#ifndef __DATABASE_HPP__
#define __DATABASE_HPP__

#include "config.h"
#include "passwd.hpp"
#include "shadow.hpp"
#include "group.hpp"
#include "gshadow.hpp"

#include <stdio.h>
#include <time.h>
#include <sys/types.h>

#define MAX_PATH		1024
#define BLOCK_SIZE	1024
#define LOCK_TRIES	  15					// Tries
#define LOCK_SLEEP	   1					// Seconds

template <class TEntry> class CDatabase
{
	public:
		CDatabase(const char *, bool, bool);
		virtual ~CDatabase();
	
	public:
		virtual int GetError() { return iErrorCode; };
		virtual bool FindUser(char *);
		virtual bool GetGroup(char *, gid_t *);
		virtual bool ChangePassword(char *, char *, int, int, int, int);
		virtual bool ChangeGecos(char *, char *);
		virtual bool DeleteEntry(char *);
		virtual bool UpdateDatabase();
		virtual char *GetPassword(char *);			// Returns the password field for each database
		virtual char *GetName(gid_t);					// For group database returns group name
																// for the parameter - group id
		virtual char *GetGecos(char *);
		virtual char *GetHome(char *);
	
		virtual void CancelUpdate();
	
		inline int GetEntries() { return iEntries; }
		inline TEntry *GetEntry(int iEntry) { return ppEntry[iEntry]; }
	
	private:
		bool CreateBackup();
		bool CreateTemp();
		bool Lock();
		bool LockNow();
		void Unlock();

	private:
		char szDatabaseName[MAX_PATH];
		char szBackupName[MAX_PATH];
		char szTempName[MAX_PATH];	
	
		bool fgLocked;
		bool fgUseLckpwdf;
	
		FILE *fpDatabase;
	
	protected:
		TEntry **ppEntry;
	
		bool fgModified;
		bool fgReadOnly;	
	
		int iEntries;
		int iErrorCode;
};

#define SHADOW_FILE "/etc/shadow"
#ifdef MASTER_PASSWD
#define PASSWD_FILE "/etc/master.passwd"
#else
#define PASSWD_FILE "/etc/passwd"
#endif
#define GROUP_FILE "/etc/group"
#define GSHADOW_FILE "/etc/gshadow"

class CShadowDatabase : public CDatabase <class CShadowEntry>
{
	public:
		CShadowDatabase(bool fgReadOnly = true);
	
	public:
		bool AddEntry(char *);
};

class CPasswdDatabase : public CDatabase <class CPasswdEntry>
{
	public:
		CPasswdDatabase(bool fgReadOnly = true);
	
	public:
		bool AddEntry(char *, unsigned int, unsigned int, char *, char *, char *);
		int GetMaxUserID();
};

class CGroupDatabase : public CDatabase <class CGroupEntry>
{
	public:
		CGroupDatabase(bool fgReadOnly = true);

	public:
		bool AddEntry(char *, unsigned int, char *);
		int GetMaxGroupID();
};

class CGshadowDatabase : public CDatabase <class CGshadowEntry>
{
	public:
		CGshadowDatabase(bool fgReadOnly = true);

	public:
		bool AddEntry(char *, char *, char *);
		bool IsAdministrator(char *, char *);
};


#endif // __DATABASE_HPP__