#ifndef __PASSWD_HPP__
#define __PASSWD_HPP__

#include <time.h>
#include <stdio.h>
#include <sys/types.h>

#include "config.h"
#include "entry.hpp"

// Structure definition
struct passwd
{
	char *pw_name;								// username
	char *pw_passwd;							// password
	uid_t pw_uid;								// user ID
	gid_t pw_gid;								// group ID
#ifdef ATT_CHANGE
	time_t pw_change;							// password change time
#endif // ATT_CHANGE
#ifdef ATT_CLASS
	char *pw_class;							// user access class
#endif // ATT_CLASS
#ifdef ATT_AGE
	char *pw_age;								// currently unused
#endif // ATT_AGE
#ifdef ATT_COMMENT	
	char *pw_comment;							// currently unused
#endif // ATT_COMMENT
	char *pw_gecos;							// Honeywell login info
	char *pw_dir;								// home directory
	char *pw_shell;							// shell program
#ifdef ATT_EXPIRE
	time_t pw_expire;							// account expiration
#endif // ATT_EXPIRE
	int pw_fields;								// fields filled in
};

class CPasswdEntry : public CEntry
{
	public:
		CPasswdEntry(FILE *);
		CPasswdEntry(char *, uid_t, gid_t, char *, char *, char *);
		~CPasswdEntry();
	
	public:
		char *GetName();
		char *GetPassword();
		char *GetGecos();
		char *GetHome();
		gid_t GetGroup();
		uid_t GetUser();
		bool SetPassword(char *, int, int, int, int);
		bool SetGecos(char *);
		bool PutEntry(FILE *);
	
		inline struct passwd *GetEntry() { return &strEntry; }

	private:
		static bool GetEntry(char *, void *);

	private:
		struct passwd strEntry;
};

#endif // __PASSWD_HPP__