/*
 * Copyright 1999, Alexander Feldman <alex@varna.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Alexander Feldman nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ALEXANDER FELDMAN AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL ALEXANDER FELDMAN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>

#include "obn.hpp"
#include "randprime.hpp"

#include "rsa.hpp"

char szPrivateKey[256];
char szPublicKey[256];

int main(int argc, char **argv)
{
	printf("Enter destination for the public key [/etc/passwdd.pbk]: ");
	fgets(szPublicKey, sizeof(szPublicKey), stdin);
	for (unsigned i = 0; i < sizeof(szPublicKey); i++)
		if (10 == szPublicKey[i] || 10 == szPublicKey[i])
			szPublicKey[i] = '\0';
	if ('\0' == szPublicKey[0])
		strcpy(szPublicKey, "/etc/passwdd.pbk");
	
	printf("Enter destination for the private key [/etc/passwdd.prk]: ");
	fgets(szPrivateKey, sizeof(szPrivateKey), stdin);
	for (unsigned i = 0; i < sizeof(szPrivateKey); i++)
		if (10 == szPrivateKey[i] || 10 == szPrivateKey[i])
			szPrivateKey[i] = '\0';
	if ('\0' == szPrivateKey[0])
		strcpy(szPrivateKey, "/etc/passwdd.prk");
	
	WORD wKeySize;
	
	do {
		printf("Select keypair key size:\n");
		printf("   1.   768 bits - commercial grade\n");
		printf("   2.  1024 bits - high commercial grade\n");
		printf("   3.  2048 bits - military grade\n");
		printf("Choose 1, 2 or 3, or enter number of bits.\n");
		printf("(768 - 2048): ");
		
		scanf("%d", &wKeySize);
		switch (wKeySize) {
			case 1:
				wKeySize = 768;
			break;
			case 2:
				wKeySize = 1024;
			break;
			case 3:
				wKeySize = 2048;
			break;
		}
	} while (wKeySize < 768 || wKeySize > 2048);

	CBigNumber pub(0x10001);
	CRandomPrimeBigNumber p(wKeySize / 2);
	while (CBigNumber::GCD(pub, p - 1) != 1)
		p.SetRandom();
	CRandomPrimeBigNumber q(wKeySize / 2);
	while (CBigNumber::GCD(pub, q - 1) != 1)
		q.SetRandom();
	CBigNumber n = p * q;
	CBigNumber phi = (p - 1) * (q - 1);
	CBigNumber sec = CBigNumber::ModInv(pub, phi);
	
	CRandomBigNumber::Destroy();
	
	FILE *fpPrivate = fopen(szPrivateKey, "w");
	if (NULL == fpPrivate) {
		fprintf(stderr, "Error opening %s...\n", szPrivateKey);
		return 1;
	}
	fprintf(fpPrivate, "-----BEGIN RSA PRIVATE KEY-----\n");
	int i;
	i = sec.Rad64Print(fpPrivate, 0);
	i = n.Rad64Print(fpPrivate, i);
	i = p.Rad64Print(fpPrivate, i);
	i = q.Rad64Print(fpPrivate, i);
	i = phi.Rad64Print(fpPrivate, i);
	if (i % 64 != 0)
		putc('\n', fpPrivate);
	fprintf(fpPrivate, "-----END RSA PRIVATE KEY-----\n");
	fclose(fpPrivate);
	
	FILE *fpPublic = fopen(szPublicKey, "w");
	if (NULL == fpPublic) {
		fprintf(stderr, "Error opening %s...\n", szPublicKey);
		return 1;
	}
	fprintf(fpPublic, "-----BEGIN RSA PUBLIC KEY-----\n");
	i = pub.Rad64Print(fpPublic, 0);
	n.Rad64Print(fpPublic, i);
	if (i % 64 != 0)
		putc('\n', fpPublic);
	fprintf(fpPublic, "-----END RSA PUBLIC KEY-----\n");
	fclose(fpPublic);

	printf("Keypair successfully created...\n");

	return 0;
}