#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/timeb.h>

#include "config.h"
#include "seed.hpp"

#define KB_GET_STATE(fd, s) tcgetattr(fd, s)
#define KB_ALTER_STATE(s) ((s)->c_cc[VMIN] = 1, (s)->c_cc[VTIME] = 0, (s)->c_lflag &= ~(ECHO | ICANON))
#define KB_SET_STATE(fd, s) tcsetattr(fd, TCSAFLUSH, s)

void GetSeed(int iBytes, unsigned char *pcData)
{
	struct termios kbState0, kbState1;
	KB_GET_STATE(0, &kbState0);
	kbState1 = kbState0;
	KB_ALTER_STATE(&kbState1);
	KB_SET_STATE(0, &kbState1);
	
	puts("Please write something meaningful (50 characters): ");
	
	while (true) {
		int c;
		read(0, &c, 1);
		if (RandPoolKeystroke(c, iBytes, pcData))
			break;
	}
	KB_SET_STATE(0, &kbState0);
	
	puts("Thanks.");
}

bool RandPoolKeystroke(int c, int iBytes, unsigned char *pcData)
{
	static int d;
	static int iEntered;
	static struct timeb t;
	static struct timeval u;
	
	if (d == c)
		return false;
	d = c;
	
	pcData[iEntered++] = (unsigned char)c;
	if (iEntered == iBytes) {
		iEntered = 0;
		return true;
	}

#ifdef HAVE_FTIME
	ftime(&t);
	pcData[iEntered++] = (unsigned char)t.millitm;
	if (iEntered == iBytes) {
		iEntered = 0;
		return true;
	}
#endif
	
	gettimeofday(&u, NULL);
	pcData[iEntered++] = (unsigned char)u.tv_usec;
	if (iEntered == iBytes) {
		iEntered = 0;
		return true;
	}
	return false;
}
